<?php

namespace Drupal\dynamic_queues\Controller;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Dynamic Queues Controller.
 */
class DynamicQueueController extends ControllerBase {

  /**
   * Configuration Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * Constructs a DynamicQueueController object.
   */
  public function __construct(ConfigFactory $configFactory) {
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $configFactory = $container->get('config.factory');
  }

  /**
   * Fetch dynamic queue count for derivative.
   */
  public static function getQueueLength($queueType = '') {
    $queue_name = 'dynamic_queues:' . $queueType . '_queue_';
    $query = \Drupal::database()->select('queue', 'q');
    $query->condition('name', "%" . \Drupal::database()->escapeLike($queue_name) . "%", 'LIKE');
    $query->condition('expire', 0);
    $query->fields('q', ['name']);
    $queue_item_exists = $query->distinct()->execute()->fetchCol();

    if (count($queue_item_exists) > 0) {
      $search = ['dynamic_queues:' . $queueType . '_queue_'];
      $replace = [''];
      $result = str_replace($search, $replace, $queue_item_exists);
      $queue_max_no = (max($result) + 1);
    }
    else {
      $queue_max_no = 0;
    }
    return $queue_max_no;
  }

  /**
   * Fetch Dynamic Queue Details from Queue table.
   */
  public static function loadDatatoDynamicQueues($queueType = '', $queueData = '') {
    $max_queue_limit = \Drupal::config('dynamic_queues.settings_form')->get('max_queue_limit');

    // Check if queue exist with queue type.
    $queue_name = 'dynamic_queues:' . $queueType . '_queue_';
    $query = \Drupal::database()->select('queue', 'q');
    $query->condition('name', "%" . \Drupal::database()->escapeLike($queue_name) . "%", 'LIKE');
    $query->condition('expire', 0);
    $query->fields('q', ['name']);
    $queue_item_exists = $query->distinct()->execute()->fetchCol();

    print_r($queue_item_exists);

    if (count($queue_item_exists) > 0) {
      $search = ['dynamic_queues:' . $queueType . '_queue_'];
      $replace = [''];
      $result = str_replace($search, $replace, $queue_item_exists);
      echo $totalQueue = (max($result) + 1);

      // Count check for queue type.
      $cqueue_name = $queue_name . max($result);
      $cquery = \Drupal::database()->select('queue', 'q');
      $cquery->condition('name', $cqueue_name, '=');
      $cquery->condition('expire', 0);
      $cquery->fields('q', ['name']);
      $cqueue_item_exists = $cquery->countQuery()->execute()->fetchCol();

      if ($cqueue_item_exists[0] == $max_queue_limit) {
        $queue_max_no = (max($result) + 1);
      }
      else {
        $queue_max_no = max($result);
      }
      $_queue_name = $queueType . "_queue_" . $queue_max_no;
    }
    else {
      $queue_max_no = 0;
      $_queue_name = $queueType . "_queue_0";
    }

    $queue = \Drupal::queue('dynamic_queues:' . $_queue_name);
    $item = new \stdClass();
    $item->data = $queueData;
    $queue->createItem($item);
    \Drupal::service('plugin.manager.queue_worker')->clearCachedDefinitions();
    return TRUE;
  }

  /**
   * Getting the results for the dashboard of dynamic queue lists.
   */
  public static function getDynamicQueueLists() {

    // Get parameter value while submitting filter form.
    $queue_lists_name = \Drupal::request()->query->get('queue_lists_name');

    // Load filter controller.
    $form['form'] = \Drupal::formBuilder()->getForm('Drupal\dynamic_queues\Form\DynamicQueuesListsFilterForm');

    if (!empty($queue_lists_name)) {
      $dataSet = self::getQueueLists($queue_lists_name);
    }
    else {
      $dataSet = "";
    }

    $header = [
      'sno' => t('Total Queues'),
      'sno' => t('S.no'),
      'title' => t('Title'),
      'emailTo' => t('To'),
      'queueStatus' => t('Status'),
      'created' => t('Created'),
    ];

    $form['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $dataSet,
      '#attributes' => ['class' => ['downloadstatistics']],
      '#empty' => t('No records found'),
    ];

    $form['pager'] = [
      '#type' => 'pager',
    ];
    return $form;
  }

  /**
   * Fetch the specific queue details from the lists.
   */
  public static function getQueueLists($queue_lists_name) {

    if ($queue_lists_name != "None") {

      $results = \Drupal::database()->select('queue', 'q');
      $results->condition('name', $queue_lists_name, '=');
      $results->extend('\Drupal\Core\Database\Query\PagerSelectExtender');
      // $results->limit(50);
      $results->fields('q');
      $results->orderBy('q.item_id', 'DESC');

      $resultset = $results->execute()->fetchAll(\PDO::FETCH_OBJ);

      $data = 0;

      foreach ($resultset as $data => $result_row) {

        if (isset($result_row->data) && !empty(isset($result_row->data))) {
          $data_unserialize = unserialize($result_row->data);
        }

        $item_data[] = $data_unserialize;

        $rows[] = [
              ['data' => $data + 1, 'width' => '5%'],
              ['data' => $item_data[$data]->data['node_title'], 'width' => '10%'],
                 ['data' => $item_data[$data]->data['emailTo'] , 'width' => '10%'],
              ['data' => ($result_row->expire == 0) ? 'In Progress' : 'Failed', 'width' => '5%'],
              ['data' => date("d-m-Y h:i:s", $result_row->created), 'width' => '5%'],
        ];
        $data++;
      }
    }
    else {
      $rows = [];
    }

    return $rows;
  }

}
